#!/bin/sh
                                                                                                                                                             
zenity --notification \
--text="There are system updates necessary!"
